#***********************************************************************
# Licensed Material - Property of IBM
#
# 5724-M24
#
# (c) Copyright IBM Corp. 2009.  All rights reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#************************************************************************

from java.lang import System
import sys

#Constants for Boolean types
True = 1 == 1
False = 1 == 0

#Flag decide whether profile migration should run or not
migrateProfile = False

TRUE = 1
FALSE = 0

# DataSource JNDI
JDBC_FABRIC_PM_JNDI = "jdbc/fabric/pm"
FABRIC_ME_JNDI_PARTIAL = "jdbc/fabricme/"
#Bus Memmber Name variables
DA_EVENT_CONNECTION_FACTORY_JNDI = "jms/fabric/DAEventConnectionFactory"
HUB_REQUEST_QUEUE_JNDI = "jms/fabric/HubRequestQueue"
DA_EVENT_TOPIC_JNDI = "jms/fabric/DAEventTopic"
DA_PERFMON_ACTIVATION_JNDI = "jms/fabric/DAPerfMonActivation"
HUB_REQUEST_ACTIVATION_JNDI = "jms/fabric/HubRequestActivation"
HUB_EVENT_ACTIVATION_JNDI = "jms/fabric/HubEventActivation"
WBSF_CONTEXT_CACHE_JNDI = "services/cache/wbsf.contexts"

#------------------------------------------------------------------------------------------
# Utility to handle list
#------------------------------------------------------------------------------------------

def wsadminToList(inStr):
	outList = []
	if (len(inStr) > 0 and inStr[0] == '[' and inStr[ - 1] == ']'):
		if(inStr[1: - 1].find("\"") >= 0):
			tmpList = splitWithQuotes(inStr[1: - 1]) #splits space/quotes seperated lists
		else:
			tmpList = inStr[1: - 1].split(' ') #splits space-separated lists,		
	else:
		tmpList = inStr.split("\n")   #splits for Windows or Linux
			
	for item in tmpList:
		item = item.rstrip();         #removes any Windows "\r"
		if (len(item) > 0):
			outList.append(item)
	return outList
#endDef

#------------------------------------------------------------------------------------------
# Update Application Ear
#------------------------------------------------------------------------------------------
def updateApp(applicationName,attr):
	try :
		print "Updating Application : ", applicationName
		AdminApp.update(applicationName, 'app', attr)
		print applicationName , " is updated successfully."
	except:
		print "Exception:Error occured while updating " + applicationName
		sys.stderr.write("Error occured while installing " + applicationName)
		raise
	#endTry :
#endDef

#------------------------------------------------------------------------------------------
#Remove Data Source
#------------------------------------------------------------------------------------------

def removeObject(fabricObjName, objList):
	for wpsobject in objList:
		objectName = AdminConfig.show(wpsobject, "jndiName")
		if(objectName.find(fabricObjName) > 0):
			AdminConfig.remove(wpsobject)
		#endIf
	#endFor
	return FALSE
#endDef

#------------------------------------------------------------------------------------------
#Get Datasource object
#------------------------------------------------------------------------------------------
def getConfigObject(fabricObjName, objList):
	for wpsobject in objList:
		objectName = AdminConfig.show(wpsobject, "jndiName")
		if(objectName.find(fabricObjName) > 0):
			#print fabricObjName," Configured" 
			return wpsobject 
		#endIf
	#endFor
	print fabricObjName, " Not Configured" 
	return None
#endDef

#-----------------------------------------------------------------
# Check if BSpace has been configured:
#-----------------------------------------------------------------
def checkIfBSpaceMigrationRequired(scope, scopePath):
	return checkDataSource("jdbc/bpm/BusinessSpace",scopePath)
	
#endDef

#------------------------------------------------------------------------------
# check whether the specified datasource is configured
#------------------------------------------------------------------------------
def checkDataSource(jndiName, scopePath):
	scope = AdminConfig.getid(scopePath)
	if not scope:
		return False
	for dataSource in AdminConfig.list("DataSource", scope).splitlines():
		if (jndiName == AdminConfig.showAttribute(dataSource, "jndiName")):
			# candidate, but may be in a sub-scope. construct id in desired scope to make sure.
			provider = AdminConfig.showAttribute(dataSource, "provider")
			providerName = AdminConfig.showAttribute(provider, "name")
			# Robustness: apparently, there may be multiple JDBC providers with the same name. In this case
			# the originally used 'AdminConfig.getid(scopePath + "JDBCProvider:" + providerName + "/DataSource:" + dataSourceName + "/")'
			# will return an invalid config id that causes problems upstream.
			for provider in AdminConfig.getid(scopePath + "JDBCProvider:" + providerName + "/").splitlines():
				for dataSource2 in AdminConfig.list("DataSource", provider).splitlines():
					if dataSource == dataSource2:
						break
				#endfor
			#endfor
			if dataSource:
				return True
			#endif
		#endif
	#endfor
	return False
#enddef


#-----------------------------------------------------------------
# Logs messages
#-----------------------------------------------------------------
def trace(message):
    print message;
#endDef

#------------------------------------------------------------------------------------------
# Update the applications
#------------------------------------------------------------------------------------------
def updateApplications(updateAppDict, appEarFolderLocation, nodeName, serverName, clusterName):
	isAppDeployed = False
	trace("************************************ Entering updateApplications ********************************")
	
	for app in updateAppDict.keys():
            appLocation = appEarFolderLocation+""+updateAppDict.get(app)            
            appAttrs = ["-operation", "update", "-contents", appLocation]

            if clusterName == '' :
                appAttrs.extend(["-node", nodeName, "-server", serverName])
            else:
	        	appAttrs.extend(["-cluster", clusterName])
            
            print "no cluster"
            updateApp(app, appAttrs)
	        
	trace("************************************ exiting updateApplications ********************************")
        
#------------------------------------------------------------------------------------------
# Install new the applications
#------------------------------------------------------------------------------------------
def installApplications(addAppDict, appEarFolderLocation, scope, nodeName, serverName, clusterName):
	isAppDeployed = False
	deployedApps = AdminApp.list(scope)
	
	for app in addAppDict.keys():
         	
         	if(deployedApps.find(app) > 0):
         		continue
         	
         	appLocation = appEarFolderLocation + addAppDict.get(app)
         	appAttrs = ["-appname", app]
         	
         	if clusterName == '':
         		appAttrs.extend(["-node", nodeName, "-server", serverName])
         	else:
         		appAttrs.extend(["-cluster", clusterName])
         		
         	print 'Installing New Application '
         	print 'Application Location :', appLocation         	
         	AdminApp.install(appLocation, appAttrs)
 	
 	return isAppDeployed    

#------------------------------------------------------------------------------------------
# Remove the applications
#------------------------------------------------------------------------------------------

def uninstallApplications(removeAppList, scope):
	appList = AdminApp.list(scope)
	for app in removeAppList:
		if(checkApplicationExist(app, scope) == True) :
			print "Removing the application : ", app
			AdminApp.uninstall(app)


#------------------------------------------------------------------------------------------
#Find target bus for Event Connection Factory
#------------------------------------------------------------------------------------------

def findBusForEventFactory(objEventConnFactory):
	propertySet = AdminConfig.showAttribute(objEventConnFactory, "propertySet")
	connFactPropertyList = AdminConfig.list("J2EEResourceProperty", propertySet).splitlines()

	for resourceList in connFactPropertyList :
		resource = resourceList
		i = resource.find('(')
		propId = resource[0:i]
		propertyName = resource[len(propId):len(resource)]
		nameList = AdminConfig.showAttribute(propertyName, "name").splitlines()
		for name in nameList :
			objectName = AdminConfig.showAttribute(propertyName, "name")
			if (objectName == "BusName") :
				objectValue = AdminConfig.showAttribute(propertyName, "value")
				print "Target Bus ", objectValue
				return objectValue
			#endIf
		#endFor	
	#endFor
	return ""

#---------------------------------------------------------------------------------------------------------------
# Searches for the Fabric Data Source using JNDI
#---------------------------------------------------------------------------------------------------------------
def getFabricDatasource(JNDI_NAME, containerID):
	dataSrcList = AdminConfig.list("DataSource", containerID).splitlines()
	
	for dataSrc in dataSrcList:
		jndiName = AdminConfig.showAttribute(dataSrc, "jndiName")
		pos = jndiName.find(JNDI_NAME)
		if(pos >= 0):
			return dataSrc

	return None

	
#------------------------------------------------------------------------------------------
# Remove Bus and JMS Config
#------------------------------------------------------------------------------------------
		
def removeBusAndJMSConfig(containerScopePath):
		print "Removing JMS Configuration..."
		
		containerID = AdminConfig.getid(containerScopePath)
		
		#------------------------------------------------------------------------------------------
		# Update DA_FACTORY 
		#------------------------------------------------------------------------------------------
		
		connFactoryList = AdminTask.listSIBJMSConnectionFactories(containerID).splitlines()
		fabricBusName = ""
		daEventConnFactory = getConfigObject(DA_EVENT_CONNECTION_FACTORY_JNDI, connFactoryList)
		if (daEventConnFactory):
			print "Removing ", daEventConnFactory
			fabricBusName = findBusForEventFactory(daEventConnFactory)
			removeObject(DA_EVENT_CONNECTION_FACTORY_JNDI, connFactoryList)
		#endIf
	
		#------------------------------------------------------------------------------------------
		# Update Queue 
		#------------------------------------------------------------------------------------------
		
		queueList = AdminTask.listSIBJMSQueues(containerID).splitlines()
		hubRequestQueue = getConfigObject(HUB_REQUEST_QUEUE_JNDI, queueList)
		if (hubRequestQueue):
			print "Removing ", hubRequestQueue
			removeObject(HUB_REQUEST_QUEUE_JNDI, queueList)
		#endIf
	
		#------------------------------------------------------------------------------------------
		# Update TOPIC 
		#------------------------------------------------------------------------------------------
		
		topicList = AdminTask.listSIBJMSTopics(containerID).splitlines()
		daEventTopic = getConfigObject(DA_EVENT_TOPIC_JNDI, topicList)
		if (daEventTopic):
			print "Removing ", daEventTopic
			removeObject(DA_EVENT_TOPIC_JNDI, topicList)
		#endIf
	
		#------------------------------------------------------------------------------------------
		# Update Activation Specs 
		#------------------------------------------------------------------------------------------
		
		activSpecList = AdminTask.listSIBJMSActivationSpecs(containerID).splitlines()
		daPerfmonActivation = getConfigObject(DA_PERFMON_ACTIVATION_JNDI, activSpecList)
		if (daPerfmonActivation):
			print "Removing ", daPerfmonActivation
			removeObject(DA_PERFMON_ACTIVATION_JNDI, activSpecList)
		#endIf
	
		activSpecList = AdminTask.listSIBJMSActivationSpecs(containerID).splitlines()
		hubRequestActivation = getConfigObject(HUB_REQUEST_ACTIVATION_JNDI, activSpecList)
		if (hubRequestActivation):
			print "Removing ", hubRequestActivation
			removeObject(HUB_REQUEST_ACTIVATION_JNDI, activSpecList)
		#endIf
	
		activSpecList = AdminTask.listSIBJMSActivationSpecs(containerID).splitlines()
		hubEventActivation = getConfigObject(HUB_EVENT_ACTIVATION_JNDI, activSpecList)
		if (hubEventActivation):
			print "Removing ", hubEventActivation
			removeObject(HUB_EVENT_ACTIVATION_JNDI, activSpecList)
		#endIf
		
		#------------------------------------------------------------------------------------------------
		# Remove the Fabric ME datasource 
		#------------------------------------------------------------------------------------------------
		fabricMEDatasource = getFabricDatasource(FABRIC_ME_JNDI_PARTIAL, containerID);
		
		if(fabricMEDatasource):
			print "Removing", fabricMEDatasource
			AdminConfig.remove(fabricMEDatasource)
		      
		#------------------------------------------------------------------------------------------ 
		# Delete Fabric Bus
		#------------------------------------------------------------------------------------------
	
		
		if(fabricBusName != "") :
			print "Searching for fabric bus :", fabricBusName;
			if AdminConfig.getid("/SIBus:" + fabricBusName + "/") :
				print "Fabric Bus Found."
				AdminTask.deleteSIBus('[-bus ' + fabricBusName + ']')
				print "Fabric Bus Removed."


#------------------------------------------------------------------------------
# Remove the specified datasource 
#------------------------------------------------------------------------------
def removeDataSource(jndiName, scopePath):
	scope = AdminConfig.getid(scopePath)
	if not scope:
		return False
	for dataSource in AdminConfig.list("DataSource", scope).splitlines():
		if (jndiName == AdminConfig.showAttribute(dataSource, "jndiName")):
			# candidate, but may be in a sub-scope. construct id in desired scope to make sure.
			provider = AdminConfig.showAttribute(dataSource, "provider")
			providerName = AdminConfig.showAttribute(provider, "name")
			# Robustness: apparently, there may be multiple JDBC providers with the same name. In this case
			# the originally used 'AdminConfig.getid(scopePath + "JDBCProvider:" + providerName + "/DataSource:" + dataSourceName + "/")'
			# will return an invalid config id that causes problems upstream.
			for provider in AdminConfig.getid(scopePath + "JDBCProvider:" + providerName + "/").splitlines():
				for dataSource2 in AdminConfig.list("DataSource", provider).splitlines():
					if dataSource == dataSource2:						
						break
				#endfor
			#endfor
			if dataSource:
				AdminConfig.remove(dataSource);
				return True
			#endif
		#endif		
	#endfor
	return False
#enddef

#------------------------------------------------------------------------------
# get the scope path according to cellname, clusterName
#------------------------------------------------------------------------------
def getClusterScopePath(cellName, clusterName):
	trace("****************getClusterScopePath() start****************");
	scopePath = "";
	if (cellName == ''):
		scopePath = '';
	elif (clusterName == ''):
		scopePath = '/Cell:' + cellName + '/';
	else:
		scopePath = '/Cell:' + cellName + '/ServerCluster:' + clusterName + '/';
	#endIf
	trace("****************getClusterScopePath() end****************");
	return scopePath;
#endDef

#------------------------------------------------------------------------------
def getScopePath(cellName, nodeName, serverName):
	trace("****************getScopePath() start****************");
	scopePath = "";
	if (cellName == ''):
		scopePath = '';
	elif (nodeName == ''):
		scopePath = '/Cell:' + cellName + '/';
	elif (serverName == ''):		
		scopePath = '/Cell:' + cellName + '/Node:' + nodeName + '/';
	else:
		scopePath = '/Cell:' + cellName + '/Node:' + nodeName + '/Server:' + serverName + '/';
	#endIf
	trace("****************getScopePath() end****************");
	return scopePath;
#endDef

#------------------------------------------------------------------------------
# get the scope Id according to cellname ,nodename, servername
#------------------------------------------------------------------------------
def getScopeId(cellName,nodeName,serverName):
	trace("****************getScopeId() start****************");
	scopeId="";
	if (cellName ==''):
		scopeId='';
	elif (nodeName ==''):
		scopeId=AdminConfig.getid('/Cell:' + cellName);
	elif (serverName == ''):
		trace(" serverName empty, getting "+ cellName + ":" + nodeName);
		scopeId=AdminConfig.getid('/Cell:' + cellName + '/Node:' + nodeName);
	else:
		scopeId=AdminConfig.getid('/Cell:' + cellName + '/Node:' +nodeName + '/Server:' + serverName);
	#endIf
	trace("****************getScopeId() end****************");
	return scopeId;
#endDef

#------------------------------------------------------------------------------
# check whether the specified application is installed 
#------------------------------------------------------------------------------
def checkApplicationExist(appName, scope):
	trace("**************** checkApplicationExist(" + appName + ") start ****************");
	result = False
	applications = AdminApp.list(scope).split()
	for application in applications:
		if (application == appName):
			result = True
			trace("**************** checkApplicationExist() end:" + str(result) + " ****************");
			return result
	trace("**************** checkApplicationExist() end:" + str(result) + " ****************");
	return result
#endDef

#-----------------------------------------------------------------
# Check if Fabric has been configured:
#-----------------------------------------------------------------
def checkIfFabricMigrationRequired(scope, scopePath):
	return checkApplicationExist("Fabric_Catalog", scope) and checkDataSource(JDBC_FABRIC_PM_JNDI, scopePath)


#-----------------------------------------------------------------------------------------------------
# update fabric rest services
#-------------------------------------------------------------------------------------------------------

def updateRestServices(restServiceTypes, nodeName, serverName, clusterName):
    
    for restServiceType in restServiceTypes :
    	attrs = ['-version', '6.2.0.0', '-enable', 'true']
        if clusterName == '':
            attrs.extend(["-nodeName", nodeName, "-serverName", serverName])
        else:
            attrs.extend(["-clusterName", clusterName])
        
        attrs.extend(['-type', restServiceType])
        
	print "Updating rest service: "+restServiceType
        
	AdminTask.updateRESTGatewayService(attrs)
		
#--------------------------------------------------------------------------------------------------------------
#     MAIN     
#---------------------------------------------------------------------------------------------------------------
def main():
	clusterName = System.getProperty("wbi.upgrade.cluster.name")
	
	if clusterName == "":
		clusterName = None
	#endif
	
	
	nodeName = System.getProperty("local.node")
	cellName = System.getProperty('local.cell')
	
	#---------------------------------------------------------------------------------------------------------------------------------
	# Refresh Fabric EARs
	#---------------------------------------------------------------------------------------------------------------------------------
	    
	appEarFolderLocation = System.getProperty("was.install.root") + "/installableApps/"
	updateAppDict = {'Fabric_Catalog':'catalog.ear', 'Fabric_Tools':'tools.ear'}
	addAppDict = {'Fabric_Trace':'trace.ear'}
	removeAppList = ['Fabric_Engine', 'Fabric_REST_Services','Fabric_Tools_Help']
	restServiceTypes = ['{com.ibm.bpm}BSLM', '{com.ibm.bpm}Glossary', '{com.ibm.bpm}Governance','{com.ibm.bpm}BVARS']
	
	if clusterName:
		trace("INFO: clusterName = " + clusterName + ".")
		scopeAtt = 'WebSphere:cell=' + cellName + ',cluster=' + clusterName
		scopePath = getClusterScopePath(cellName, clusterName)
		cellPath  = getClusterScopePath(cellName,'')

		
		if (checkIfBSpaceMigrationRequired(scopeAtt,scopePath)):
		
		    #Enable Fabric REST services on cluster.
		    try :
			updateRestServices(restServiceTypes,'', '', clusterName)
			print "Successfully updated fabric rest services on ." + clusterName
                    except:
			print "Failed to update fabric rest services."

                    #Save the REST configuration
		    AdminConfig.save()
		
		
		if (not checkIfFabricMigrationRequired(scopeAtt, cellPath)):
				trace("No Fabric configured in target cluster, exit Fabric migration in target cluster");
				return;
		
		updateApplications(updateAppDict, appEarFolderLocation, '', '', clusterName);
		installApplications(addAppDict, appEarFolderLocation, scopeAtt, '', '', clusterName)
		uninstallApplications(removeAppList, scopeAtt)
		removeDataSource(JDBC_FABRIC_PM_JNDI, cellPath)
		removeBusAndJMSConfig(scopePath)
		#Enable Fabric REST services on cluster.
		try :
		    updateRestServices(restServiceTypes,'', '', clusterName)
		    print "Successfully updated fabric rest services on ." + clusterName
                except:
	            print "Failed to update fabric rest services."
          
	
	else :
		trace("INFO: no cluster")		
		nodeId = getScopeId(cellName, nodeName, '')
		print "nodeId is: " + str(nodeId)
		serverList = wsadminToList(AdminConfig.list('Server', nodeId))
		for server in serverList:		
			serverName = AdminConfig.showAttribute(server, 'name')
			trace('serverName = ' + str(serverName))
			scopeAtt = 'WebSphere:cell=' + cellName + ',node=' + nodeName + ',server=' + serverName
			scopePath = getScopePath(cellName, nodeName, serverName)
			if (not checkIfFabricMigrationRequired(scopeAtt, scopePath)):
				trace("No Fabric configured in target server, exit Fabric migration in target server");
				continue;
			updateApplications(updateAppDict, appEarFolderLocation, nodeName, serverName, '');
			installApplications(addAppDict, appEarFolderLocation, scopeAtt, nodeName, serverName, '')
			uninstallApplications(removeAppList, scopeAtt)
			removeDataSource(JDBC_FABRIC_PM_JNDI, scopePath)
			removeBusAndJMSConfig(scopePath)
			
			try :
				updateRestServices(restServiceTypes, nodeName, serverName, '')
				print "Successfully updated fabric rest services."
			except:
				print "Failed to update fabric rest services."
			
			try :
				AdminTask.registerRESTServiceEndpoint(['-businessSpaceNodeName', nodeName, '-businessSpaceServerName', serverName])
				print "Successfully registered fabric rest services."
			except:
				print "Failed to register fabric rest services with business space."
	
	AdminConfig.save()
	
	print "Synchronizing changes with all the nodes in cluster."
	
	#-----------------------------------------------------------------------------------------------------------------------------------
	#Sync node
	#------------------------------------------------------------------------------------------------------------------------------------
	
#	nodeList = AdminConfig.list('Node').splitlines()
#	for n in nodeList:
#		nodeid = n
#		i = nodeid.find('(')
#		nodename = nodeid[0:i]
#		objn = "type=NodeSync,node=" + nodename + ",*"
#		try :
#			Sync = AdminControl.completeObjectName(objn)
#			if Sync != "":
#				AdminControl.invoke(Sync, 'sync')
#				print "Done with synchronization for node " + nodename
#			continue
#		except :
#			print "AdminControl not available.Skipping " + nodename + " synchronization."
#		#endTry
#	#endFor
#	#Sync code complete
	
#-----------------------------------------------------------------
# Start Main function
#-----------------------------------------------------------------
main();
#-----------------------------------------------------------------
# End Main function
#-----------------------------------------------------------------